/*
 * Copyright (C) 2016 Jolla Ltd
 * Contact: Andrew den Exter <andrew.den.exter@jolla.com>
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Nemo Mobile nor the names of its contributors
 *     may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 */

#ifndef NEMODEVICELOCK_AUTHENTICATION_H
#define NEMODEVICELOCK_AUTHENTICATION_H

#include <nemo-devicelock/authenticator.h>

#include <QVariant>

namespace NemoDeviceLock {

class NEMODEVICELOCK_EXPORT Authorization : public QObject
{
    Q_OBJECT
    Q_ENUMS(Status)
    Q_FLAGS(NemoDeviceLock::Authenticator::Methods)
    Q_PROPERTY(NemoDeviceLock::Authenticator::Methods allowedMethods READ allowedMethods NOTIFY allowedMethodsChanged)
    Q_PROPERTY(Status status READ status NOTIFY statusChanged)
    Q_PROPERTY(QVariant challengeCode READ challengeCode NOTIFY challengeCodeChanged)
public:
    enum Status {
        NoChallenge,
        RequestingChallenge,
        ChallengeIssued
    };

    explicit Authorization(QObject *parent = nullptr);
    ~Authorization();

    virtual Authenticator::Methods allowedMethods() const = 0;

    virtual Status status() const = 0;
    virtual QVariant challengeCode() const = 0;

    Q_INVOKABLE virtual void requestChallenge() = 0;
    Q_INVOKABLE virtual void relinquishChallenge() = 0;

signals:
    void challengeIssued();
    void challengeDeclined();
    void challengeExpired();

    void statusChanged();
    void allowedMethodsChanged();
    void challengeCodeChanged();
};

}

#endif

